package servlet;

import enums.FileType;
import enums.MethodEnum;
import enums.ReturnCodes;
import org.blinkenlights.jid3.ID3Exception;
import org.xml.sax.SAXException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import server.Request;
import util.*;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.security.NoSuchAlgorithmException;

public class ProcessAction {

    private static Log LOG = LogFactory.getLog(ProcessAction.class);

    public static void handleMP3Request(DataOutputStream output, MethodEnum method, Request request) throws IOException, ID3Exception {
        LOG.info("handling mp3 request");

        BufferedInputStream bufferedInputStream = null;
        File currentMP3 = null;
        File coverArt = null;
        try {
            currentMP3 = Mp3Processor.findCurrentMP3();
            String title = request.getParamter(Constants.PARAM_TITLE);
            String artist = request.getParamter(Constants.PARAM_ARTIST);
            String stationId = request.getParamter(Constants.PARAM_STATION_ID);
            String pandoraUserName = request.getParamter(Constants.PARAM_PANDORA_USER_NAME);
            String cddbLookUp = request.getParamter(Constants.PARAM_CDDB_LOOKUP);
            SongInfo songInfo = new SongInfo(artist, title);

            if (cddbLookUp != null && cddbLookUp.equals("on")) {
                try {
                    LOG.info("looking up ID3 tag info 4 [" + artist + "|" + title + "]");
                    CddbLookUp.lookup(songInfo);
                } catch (Exception e) {
                    LOG.warn("failed lookup -- no biggy: " + e.getMessage());
                    //ignore we can do without them
                }
            }

            if (stationId != null && pandoraUserName != null) {
                try {
                    songInfo.setStationName(LastFmLookup.getStationName(pandoraUserName, stationId));
                } catch (Exception e) {
                    LOG.warn("unable to lookup station name", e);
                }
            }

            request.getSession().setAttribute(Constants.ATTR_SONG_INFO, songInfo);
            String albumImgLoc = request.getParamter(Constants.PARAM_ALBUM_IMG_LOC);
            if (songInfo.getAlbum() != null && albumImgLoc != null) {
                try {
                    coverArt = saveAlbumImage(songInfo, albumImgLoc);
                } catch (Exception e) {
                    LOG.warn("unable to save img - albumImgLoc: " + albumImgLoc + " songInfo: " + songInfo + ". error: " + e.getMessage());
                }
            } else {
                LOG.warn("unable to save img - albumImgLoc: " + albumImgLoc + " songInfo: " + songInfo);
            }

            Mp3Processor.addID3Tags(currentMP3, coverArt, request.getParamter(Constants.PARAM_COMMENT), songInfo);

            LOG.debug("cruise on - dealing with file manualy");
            output.writeBytes(Util.writeResponseHeader(ReturnCodes.OK, FileType.HTML, null));
            Mp3Processor.saveMP3(songInfo, currentMP3, FileType.MP3, output);

//            else {
//                output.writeBytes(writeResponseHeader(ReturnCodes.OK, FileType.MP3, title + ".mp3"));
//                bufferedInputStream = new BufferedInputStream(new FileInputStream(currentMP3));
//                byte[] buffer = new byte[256];
//                if (method != MethodEnum.HEAD) {
//                    while (bufferedInputStream.read(buffer) != -1) {
//                        output.write(buffer);
//                    }
//                }
//            }
        } finally {
            if (bufferedInputStream != null) {
                bufferedInputStream.close();
            }
            if (currentMP3 != null) {
                currentMP3.delete();
            }
        }
    }

    public static void handleStaticRequest(DataOutputStream output, MethodEnum method, String url) throws IOException {
        LOG.info("processing static request. path: " + url);
        String path = url.substring(url.indexOf("/") + 1, url.indexOf(" HTTP"));
        LOG.info("path = " + path);
        if (path.trim().length() == 0) {
            path = Constants.DEFAULT_PAGE;
        }
        LOG.debug("using path: " + path);
        FileInputStream fileInputStream = null;

        try {
            int beginIndex = path.lastIndexOf(".");
            if (beginIndex == -1) {
                throw new RuntimeException("Invalid request: " + path);
            }

            String documentType = (path.substring(beginIndex + 1)).toUpperCase();
            LOG.info("documentType = " + documentType);
            FileType fileType = FileType.valueOf(documentType);
            LOG.info("fileType = " + fileType);

            output.writeBytes(Util.writeResponseHeader(ReturnCodes.OK, fileType, null));
            if (method == MethodEnum.GET || method == MethodEnum.POST) {
                fileInputStream = new FileInputStream(path);
                Util.writeBinary(fileInputStream, output);
            }
        } catch (IOException e) {
            output.writeBytes(Util.writeResponseHeader(ReturnCodes.NOT_FOUND, FileType.HTML, null));
            output.writeBytes("file not found");
        }
        finally {
            if (fileInputStream != null) {
                fileInputStream.close();
            }
        }
    }

    public static void getArtistBio(Request request, DataOutputStream output) throws IOException, SAXException {
        BufferedReader bufferedReader = null;
        try {
            String bio = LastFmLookup.getBio(request.getParamter(Constants.PARAM_ARTIST));
            bufferedReader = new BufferedReader(new StringReader(bio));
            Util.writeFile(bufferedReader, output);
        } finally {
            if (bufferedReader != null) {
                bufferedReader.close();
            }
        }
    }

    public static void getTopFans(Request request, DataOutputStream output) throws IOException, SAXException {
        BufferedReader bufferedReader = null;
        try {
            String bio = LastFmLookup.getTopFans(request.getParamter(Constants.PARAM_ARTIST), request.getParamter(Constants.PARAM_TITLE));
            bufferedReader = new BufferedReader(new StringReader(bio));
            Util.writeFile(bufferedReader, output);
        } finally {
            if (bufferedReader != null) {
                bufferedReader.close();
            }
        }
    }

    public static void addTrackToLastFm(Request request, DataOutputStream response) throws IOException, SAXException, NoSuchAlgorithmException {
        LastFmLookup.addTrack(request);
        response.writeBytes("successfully added track to last.fm");
    }

    public static void getStats(Request request, DataOutputStream response) throws IOException {
        try {
            LastFmTrackStats lastFmTrackStat = LastFmLookup.getTrackStat(request.getParamter(Constants.PARAM_ARTIST), request.getParamter(Constants.PARAM_TITLE));
            StringBuilder sb = new StringBuilder();
            sb.append("<table>");
            if (lastFmTrackStat.getAlbumImageLocation() != null) {
                sb.append("<tr><td>").append("<img src=\"")
                        .append(lastFmTrackStat.getAlbumImageLocation())
                        .append("\" border=\"0\"/><br/>")
                        .append("<input name=\"albumImgLocation\" id=\"albumImgLocation\" type=\"hidden\" value=\"")
                        .append(lastFmTrackStat.getAlbumImageLocation())
                        .append("\"/>");
            }
            sb.append("</td></tr>")
                    .append(lastFmTrackStat.getTrackStats())
                    .append("</td></tr><tr><td>").append(lastFmTrackStat.getSimilarArtist())
                    .append("</tr></td></table>");
            response.writeBytes(sb.toString());
        } catch (Exception e) {
            response.writeBytes("No song stats found for this artist");
        }
    }

    public static void addTrackToLastFm(Request request, SongInfo songInfo) throws NoSuchAlgorithmException, IOException, SAXException {
        String password = request.getParamter(Constants.PARAM_LAST_FM_PASSWORD);
        String userName = request.getParamter(Constants.PARAM_LAST_FM_USER_NAME);

        LOG.debug("password [" + password + "]");
        LOG.debug("userName [" + userName + "]");

        if (password == null || userName == null) {
            throw new RuntimeException("unable to authenticate to last.fm");
        }

        //LastFmLookup.addTrack(userName, password, songInfo);
        LOG.info("successfully addd track: " + songInfo.getTitle() + "\n\n");
    }

    public static File saveAlbumImage(SongInfo songInfo, String imgLocation) throws IOException {
        //HttpURLConnection.setFollowRedirects(false);
        File coverArt;

        int index = imgLocation.lastIndexOf(".");
        if (index == -1) {
            throw new RuntimeException("invalid coverArt name: " + imgLocation);
        }

        String fileTypeString = imgLocation.substring(index + 1);
        FileType fileType = FileType.valueOf(fileTypeString.toUpperCase());

        URLConnection conn;
        HttpURLConnection httpConn = null;
        FileOutputStream fileOutputStream = null;
        try {
            URL url = new URL(imgLocation);
            conn = url.openConnection();

            httpConn = (HttpURLConnection) conn;
            httpConn.setInstanceFollowRedirects(false);
            conn.connect();

            InputStream in = conn.getInputStream();
            File songHierarchy = Util.createSongHierarchy(songInfo);
            coverArt = new File(songHierarchy, SafeString.getSafeFileName(songInfo.getAlbum()) + "." + fileType.toString().toLowerCase());
            fileOutputStream = new FileOutputStream(coverArt);
            byte[] buf = new byte[32 * 1024];
            int i;
            while ((i = in.read(buf)) != -1) {
                fileOutputStream.write(buf, 0, i);
            }
            LOG.info("saved album image: " + coverArt.getAbsolutePath());
        }
        finally {
            if (fileOutputStream != null) {
                fileOutputStream.flush();
                fileOutputStream.close();
            }
            if (httpConn != null) {
                httpConn.disconnect();
            }

        }

        return coverArt;
    }
}
